@@ -40,8 +40,8 @@
  *
  * You should have received a copy of the GNU Library General Public
  * License along with this library; if not, write to the
- * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
- * Boston, MA 02111-1307, USA.
+ * Free Software Foundation, Inc., 51 Franklin St, Fifth Floor,
+ * Boston, MA 02110-1301, USA.
  */
 
 /**
@@ -75,6 +75,7 @@
 #endif
 
 #include <gst/gst.h>
+#include <gst/video/gstvideometa.h>
 
 #include "gstopencvutils.h"
 #include "gstfacedetect.h"
@@ -669,6 +670,8 @@ gst_face_detect_transform_ip (GstOpencvVideoFilter * base, GstBuffer * buf,
               1, 8, 0);
         }
       }
+      gst_buffer_add_video_region_of_interest_meta (buf, "face",
+          (guint) r->x, (guint) r->y, (guint) r->width, (guint) r->height);
     }
 
     gst_structure_set_value ((GstStructure *) gst_message_get_structure (msg),
