@@ -234,9 +234,7 @@ static void
 gst_inter_audio_src_get_times (GstBaseSrc * src, GstBuffer * buffer,
     GstClockTime * start, GstClockTime * end)
 {
-  GstInterAudioSrc *interaudiosrc = GST_INTER_AUDIO_SRC (src);
-
-  GST_DEBUG_OBJECT (interaudiosrc, "get_times");
+  GST_DEBUG_OBJECT (src, "get_times");
 
   /* for live sources, sync on the timestamp of the buffer */
   if (gst_base_src_is_live (src)) {
@@ -272,7 +270,7 @@ gst_inter_audio_src_create (GstBaseSrc * src, guint64 offset, guint size,
 
   buffer = NULL;
 
-  g_mutex_lock (interaudiosrc->surface->mutex);
+  g_mutex_lock (&interaudiosrc->surface->mutex);
   n = gst_adapter_available (interaudiosrc->surface->audio_adapter) / 4;
   if (n > SIZE * 3) {
     GST_WARNING ("flushing %d samples", SIZE / 2);
@@ -285,7 +283,7 @@ gst_inter_audio_src_create (GstBaseSrc * src, guint64 offset, guint size,
     buffer = gst_adapter_take_buffer (interaudiosrc->surface->audio_adapter,
         n * 4);
   }
-  g_mutex_unlock (interaudiosrc->surface->mutex);
+  g_mutex_unlock (&interaudiosrc->surface->mutex);
 
   if (n < SIZE) {
     GstBuffer *newbuf = gst_buffer_new_and_alloc ((SIZE - n) * 4);
@@ -325,10 +323,9 @@ gst_inter_audio_src_create (GstBaseSrc * src, guint64 offset, guint size,
 static gboolean
 gst_inter_audio_src_query (GstBaseSrc * src, GstQuery * query)
 {
-  GstInterAudioSrc *interaudiosrc = GST_INTER_AUDIO_SRC (src);
   gboolean ret;
 
-  GST_DEBUG_OBJECT (interaudiosrc, "query");
+  GST_DEBUG_OBJECT (src, "query");
 
   switch (GST_QUERY_TYPE (query)) {
     case GST_QUERY_LATENCY:{
@@ -360,15 +357,14 @@ gst_inter_audio_src_query (GstBaseSrc * src, GstQuery * query)
 static GstCaps *
 gst_inter_audio_src_fixate (GstBaseSrc * src, GstCaps * caps)
 {
-  GstInterAudioSrc *interaudiosrc = GST_INTER_AUDIO_SRC (src);
   GstStructure *structure;
 
+  GST_DEBUG_OBJECT (src, "fixate");
+
   caps = gst_caps_make_writable (caps);
 
   structure = gst_caps_get_structure (caps, 0);
 
-  GST_DEBUG_OBJECT (interaudiosrc, "fixate");
-
   gst_structure_fixate_field_nearest_int (structure, "channels", 2);
   gst_structure_fixate_field_nearest_int (structure, "rate", 48000);
 
