@@ -20,8 +20,8 @@
  *
  * You should have received a copy of the GNU Library General Public
  * License along with this library; if not, write to the
- * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
- * Boston, MA 02111-1307, USA.
+ * Free Software Foundation, Inc., 51 Franklin St, Fifth Floor,
+ * Boston, MA 02110-1301, USA.
  */
 
 #ifdef HAVE_CONFIG_H
@@ -39,21 +39,23 @@ GST_DEBUG_CATEGORY (mpeg4v_parse_debug);
 #define GST_CAT_DEFAULT mpeg4v_parse_debug
 
 static GstStaticPadTemplate src_template =
-GST_STATIC_PAD_TEMPLATE ("src", GST_PAD_SRC,
+    GST_STATIC_PAD_TEMPLATE ("src", GST_PAD_SRC,
     GST_PAD_ALWAYS,
     GST_STATIC_CAPS ("video/mpeg, "
         "mpegversion = (int) 4, "
         "width = (int)[ 0, max ], "
         "height = (int)[ 0, max ], "
         "framerate = (fraction)[ 0, max ] ,"
-        "parsed = (boolean) true, " "systemstream = (boolean) false")
+        "parsed = (boolean) true, " "systemstream = (boolean) false; "
+        "video/x-divx, " "divxversion = (int) [ 4, 5 ]")
     );
 
 static GstStaticPadTemplate sink_template =
-GST_STATIC_PAD_TEMPLATE ("sink", GST_PAD_SINK,
+    GST_STATIC_PAD_TEMPLATE ("sink", GST_PAD_SINK,
     GST_PAD_ALWAYS,
     GST_STATIC_CAPS ("video/mpeg, "
-        "mpegversion = (int) 4, " "systemstream = (boolean) false")
+        "mpegversion = (int) 4, " "systemstream = (boolean) false; "
+        "video/x-divx, " "divxversion = (int) [ 4, 5 ]")
     );
 
 /* Properties */
@@ -273,7 +275,7 @@ gst_mpeg4vparse_process_config (GstMpeg4VParse * mp4vparse,
       mp4vparse->vol.fixed_vop_time_increment);
 
 
-  GST_LOG_OBJECT (mp4vparse, "accepting parsed config size %" G_GSSIZE_FORMAT,
+  GST_LOG_OBJECT (mp4vparse, "accepting parsed config size %" G_GSIZE_FORMAT,
       size);
 
   if (mp4vparse->config != NULL)
@@ -287,44 +289,6 @@ gst_mpeg4vparse_process_config (GstMpeg4VParse * mp4vparse,
   return TRUE;
 }
 
-static gboolean
-gst_mpeg4vparse_get_vop_coded (GstMpeg4VParse * mp4vparse, const guint8 * data,
-    gint vop_offset, gsize size, gsize frame_size)
-{
-  if (frame_size > 9) {         /* assuming bigger frame will always have vop_coded (saves some parsing) */
-    return TRUE;
-  } else if (size > vop_offset + 3) {
-    GstBitReader reader;
-    guint8 value;
-
-    gst_bit_reader_init (&reader, data + vop_offset + 1, size - vop_offset);
-    gst_bit_reader_skip (&reader, 2);   /* VOP_coding_type */
-
-    /* modulo_time_base (ends with 0) */
-    while (gst_bit_reader_get_bits_uint8 (&reader, &value, 1) && value);
-
-    /* marker_bit */
-    g_return_val_if_fail (gst_bit_reader_get_bits_uint8 (&reader, &value, 1)
-        && value, TRUE);
-
-    /* VOP_time_increment */
-    gst_bit_reader_skip (&reader, mp4vparse->vol.vop_time_increment_bits);
-
-    /* marker_bit */
-    g_return_val_if_fail (gst_bit_reader_get_bits_uint8 (&reader, &value, 1)
-        && value, TRUE);
-
-    /* VOP_coded */
-    if (!gst_bit_reader_get_bits_uint8 (&reader, &value, 1)) {
-      return FALSE;
-    }
-
-    return value;
-  }
-
-  return FALSE;
-}
-
 /* caller guarantees at least start code in @buf at @off */
 static gboolean
 gst_mpeg4vparse_process_sc (GstMpeg4VParse * mp4vparse, GstMpeg4Packet * packet,
@@ -344,12 +308,8 @@ gst_mpeg4vparse_process_sc (GstMpeg4VParse * mp4vparse, GstMpeg4Packet * packet,
       GST_WARNING_OBJECT (mp4vparse, "no data following VOP startcode");
       mp4vparse->intra_frame = FALSE;
     }
-    mp4vparse->vop_coded =
-        gst_mpeg4vparse_get_vop_coded (mp4vparse, packet->data,
-        mp4vparse->vop_offset, size, packet->offset - 3);
-    GST_LOG_OBJECT (mp4vparse,
-        "ending frame of size %d, is intra %d, vop_coded %d",
-        packet->offset - 3, mp4vparse->intra_frame, mp4vparse->vop_coded);
+    GST_LOG_OBJECT (mp4vparse, "ending frame of size %d, is intra %d",
+        packet->offset - 3, mp4vparse->intra_frame);
     return TRUE;
   }
 
@@ -429,7 +389,7 @@ gst_mpeg4vparse_handle_frame (GstBaseParse * parse,
   gsize size;
   gint off = 0;
   gboolean ret = FALSE;
-  guint framesize;
+  guint framesize = 0;
 
   gst_buffer_map (frame->buffer, &map, GST_MAP_READ);
   data = map.data;
@@ -570,11 +530,11 @@ gst_mpeg4vparse_update_src_caps (GstMpeg4VParse * mp4vparse)
     s = gst_caps_get_structure (caps, 0);
   } else {
     caps = gst_caps_new_simple ("video/mpeg",
-        "mpegversion", G_TYPE_INT, 4, NULL);
+        "mpegversion", G_TYPE_INT, 4,
+        "systemstream", G_TYPE_BOOLEAN, FALSE, NULL);
   }
 
-  gst_caps_set_simple (caps, "systemstream", G_TYPE_BOOLEAN, FALSE,
-      "parsed", G_TYPE_BOOLEAN, TRUE, NULL);
+  gst_caps_set_simple (caps, "parsed", G_TYPE_BOOLEAN, TRUE, NULL);
 
   if (mp4vparse->profile && mp4vparse->level) {
     gst_caps_set_simple (caps, "profile", G_TYPE_STRING, mp4vparse->profile,
@@ -592,17 +552,23 @@ gst_mpeg4vparse_update_src_caps (GstMpeg4VParse * mp4vparse)
   }
 
   /* perhaps we have a framerate */
-  if (mp4vparse->vol.fixed_vop_time_increment != 0 &&
-      (!s || !gst_structure_has_field (s, "framerate"))) {
+  {
     gint fps_num = mp4vparse->vol.vop_time_increment_resolution;
     gint fps_den = mp4vparse->vol.fixed_vop_time_increment;
-    GstClockTime latency = gst_util_uint64_scale (GST_SECOND, fps_den, fps_num);
+    GstClockTime latency;
 
-    gst_caps_set_simple (caps, "framerate",
-        GST_TYPE_FRACTION, fps_num, fps_den, NULL);
-    gst_base_parse_set_frame_rate (GST_BASE_PARSE (mp4vparse),
-        fps_num, fps_den, 0, 0);
-    gst_base_parse_set_latency (GST_BASE_PARSE (mp4vparse), latency, latency);
+    /* upstream overrides */
+    if (s && gst_structure_has_field (s, "framerate"))
+      gst_structure_get_fraction (s, "framerate", &fps_num, &fps_den);
+
+    if (fps_den > 0 && fps_num > 0) {
+      gst_caps_set_simple (caps, "framerate",
+          GST_TYPE_FRACTION, fps_num, fps_den, NULL);
+      gst_base_parse_set_frame_rate (GST_BASE_PARSE (mp4vparse),
+          fps_num, fps_den, 0, 0);
+      latency = gst_util_uint64_scale (GST_SECOND, fps_den, fps_num);
+      gst_base_parse_set_latency (GST_BASE_PARSE (mp4vparse), latency, latency);
+    }
   }
 
   /* or pixel-aspect-ratio */
@@ -613,6 +579,10 @@ gst_mpeg4vparse_update_src_caps (GstMpeg4VParse * mp4vparse)
         mp4vparse->vol.par_height, NULL);
   }
 
+  if (mp4vparse->vol.sprite_enable != GST_MPEG4_SPRITE_UNUSED)
+    gst_caps_set_simple (caps, "sprite-warping-points", G_TYPE_INT,
+        mp4vparse->vol.no_of_sprite_warping_points, NULL);
+
   gst_pad_set_caps (GST_BASE_PARSE_SRC_PAD (mp4vparse), caps);
   gst_caps_unref (caps);
 
@@ -632,9 +602,6 @@ gst_mpeg4vparse_parse_frame (GstBaseParse * parse, GstBaseParseFrame * frame)
   else
     GST_BUFFER_FLAG_SET (buffer, GST_BUFFER_FLAG_DELTA_UNIT);
 
-  if (!mp4vparse->vop_coded)    /* buffer without VOP_coded has no data */
-    GST_BUFFER_DURATION (buffer) = 0;
-
   if (G_UNLIKELY (mp4vparse->drop && !mp4vparse->config)) {
     GST_LOG_OBJECT (mp4vparse, "dropping frame as no config yet");
     return GST_BASE_PARSE_FLOW_DROPPED;
@@ -846,7 +813,7 @@ gst_mpeg4vparse_get_caps (GstBaseParse * parse, GstCaps * filter)
   GstCaps *res;
 
   templ = gst_pad_get_pad_template_caps (GST_BASE_PARSE_SINK_PAD (parse));
-  peercaps = gst_pad_get_allowed_caps (GST_BASE_PARSE_SRC_PAD (parse));
+  peercaps = gst_pad_peer_query_caps (GST_BASE_PARSE_SRC_PAD (parse), filter);
 
   if (peercaps) {
     guint i, n;
@@ -862,6 +829,7 @@ gst_mpeg4vparse_get_caps (GstBaseParse * parse, GstCaps * filter)
 
     res = gst_caps_intersect_full (peercaps, templ, GST_CAPS_INTERSECT_FIRST);
     gst_caps_unref (peercaps);
+    res = gst_caps_make_writable (res);
 
     /* Append the template caps because we still want to accept
      * caps without any fields in the case upstream does not
