@@ -18,8 +18,8 @@
  *
  * You should have received a copy of the GNU Library General Public
  * License along with this library; if not, write to the
- * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
- * Boston, MA 02111-1307, USA.
+ * Free Software Foundation, Inc., 51 Franklin St, Fifth Floor,
+ * Boston, MA 02110-1301, USA.
  */
 
 #ifdef HAVE_CONFIG_H
@@ -28,6 +28,7 @@
 
 #include <string.h>
 #include <gst/base/gstbytereader.h>
+#include <gst/codecparsers/gstmpegvideometa.h>
 
 #include "gstmpegvideoparse.h"
 
@@ -75,6 +76,8 @@ static GstCaps *gst_mpegv_parse_get_caps (GstBaseParse * parse,
     GstCaps * filter);
 static GstFlowReturn gst_mpegv_parse_pre_push_frame (GstBaseParse * parse,
     GstBaseParseFrame * frame);
+static gboolean gst_mpegv_parse_sink_query (GstBaseParse * parse,
+    GstQuery * query);
 
 static void gst_mpegv_parse_set_property (GObject * object, guint prop_id,
     const GValue * value, GParamSpec * pspec);
@@ -164,6 +167,7 @@ gst_mpegv_parse_class_init (GstMpegvParseClass * klass)
   parse_class->get_sink_caps = GST_DEBUG_FUNCPTR (gst_mpegv_parse_get_caps);
   parse_class->pre_push_frame =
       GST_DEBUG_FUNCPTR (gst_mpegv_parse_pre_push_frame);
+  parse_class->sink_query = GST_DEBUG_FUNCPTR (gst_mpegv_parse_sink_query);
 }
 
 static void
@@ -185,6 +189,8 @@ gst_mpegv_parse_reset_frame (GstMpegvParse * mpvparse)
   mpvparse->frame_repeat_count = 0;
   memset (mpvparse->ext_offsets, 0, sizeof (mpvparse->ext_offsets));
   mpvparse->ext_count = 0;
+  mpvparse->slice_count = 0;
+  mpvparse->slice_offset = 0;
 }
 
 static void
@@ -194,12 +200,40 @@ gst_mpegv_parse_reset (GstMpegvParse * mpvparse)
   mpvparse->profile = 0;
   mpvparse->update_caps = TRUE;
   mpvparse->send_codec_tag = TRUE;
+  mpvparse->send_mpeg_meta = TRUE;
 
   gst_buffer_replace (&mpvparse->config, NULL);
   memset (&mpvparse->sequencehdr, 0, sizeof (mpvparse->sequencehdr));
   memset (&mpvparse->sequenceext, 0, sizeof (mpvparse->sequenceext));
   memset (&mpvparse->sequencedispext, 0, sizeof (mpvparse->sequencedispext));
   memset (&mpvparse->pichdr, 0, sizeof (mpvparse->pichdr));
+  memset (&mpvparse->picext, 0, sizeof (mpvparse->picext));
+
+  mpvparse->seqhdr_updated = FALSE;
+  mpvparse->seqext_updated = FALSE;
+  mpvparse->seqdispext_updated = FALSE;
+  mpvparse->picext_updated = FALSE;
+  mpvparse->quantmatrext_updated = FALSE;
+}
+
+static gboolean
+gst_mpegv_parse_sink_query (GstBaseParse * parse, GstQuery * query)
+{
+  gboolean res;
+  GstMpegvParse *mpvparse = GST_MPEGVIDEO_PARSE (parse);
+
+  res = GST_BASE_PARSE_CLASS (parent_class)->sink_query (parse, query);
+
+  if (res && GST_QUERY_TYPE (query) == GST_QUERY_ALLOCATION) {
+    mpvparse->send_mpeg_meta =
+        gst_query_find_allocation_meta (query, GST_MPEG_VIDEO_META_API_TYPE,
+        NULL);
+
+    GST_DEBUG_OBJECT (parse, "Downstream can handle GstMpegVideo GstMeta : %d",
+        mpvparse->send_mpeg_meta);
+  }
+
+  return res;
 }
 
 static gboolean
@@ -229,13 +263,12 @@ gst_mpegv_parse_stop (GstBaseParse * parse)
 }
 
 static gboolean
-gst_mpegv_parse_process_config (GstMpegvParse * mpvparse, GstBuffer * buf,
+gst_mpegv_parse_process_config (GstMpegvParse * mpvparse, GstMapInfo * info,
     guint size)
 {
-  guint8 *data;
+  GstMpegVideoPacket packet;
   guint8 *data_with_prefix;
-  GstMapInfo map;
-  gint i, offset;
+  gint i;
 
   if (mpvparse->seq_offset < 4) {
     /* This shouldn't happen, but just in case... */
@@ -243,49 +276,67 @@ gst_mpegv_parse_process_config (GstMpegvParse * mpvparse, GstBuffer * buf,
     return FALSE;
   }
 
-  gst_buffer_map (buf, &map, GST_MAP_READ);
-  data = map.data + mpvparse->seq_offset;
-  g_assert (size <= map.size);
+  packet.data = info->data;
+  packet.type = GST_MPEG_VIDEO_PACKET_SEQUENCE;
+  packet.offset = mpvparse->seq_offset;
+  packet.size = size - mpvparse->seq_offset;
   /* pointer to sequence header data including the start code prefix -
      used for codec private data */
-  data_with_prefix = data - 4;
+  data_with_prefix = (guint8 *) packet.data + packet.offset - 4;
 
-  /* only do stuff if something new; only compare first 11 bytes, changes in
-     quantiser matrix doesn't matter here. Also changing the matrices in
-     codec_data seems to cause problem with decoders */
-  if (mpvparse->config && size == gst_buffer_get_size (mpvparse->config) &&
+  /* only do stuff if something new; only compare first 8 bytes, changes in
+     quantiser matrix or bitrate don't matter here. Also changing the
+     matrices in codec_data seems to cause problem with decoders */
+  if (mpvparse->config &&
       gst_buffer_memcmp (mpvparse->config, 0, data_with_prefix, MIN (size,
-              11)) == 0) {
-    gst_buffer_unmap (buf, &map);
+              8)) == 0) {
     return TRUE;
   }
 
-  if (!gst_mpeg_video_parse_sequence_header (&mpvparse->sequencehdr, data,
-          size - mpvparse->seq_offset, 0)) {
+  if (!gst_mpeg_video_packet_parse_sequence_header (&packet,
+          &mpvparse->sequencehdr)) {
     GST_DEBUG_OBJECT (mpvparse,
         "failed to parse config data (size %d) at offset %d",
         size, mpvparse->seq_offset);
-    gst_buffer_unmap (buf, &map);
     return FALSE;
   }
 
+  mpvparse->seqhdr_updated = TRUE;
+
   GST_LOG_OBJECT (mpvparse, "accepting parsed config size %d", size);
 
   /* Set mpeg version, and parse sequence extension */
   mpvparse->config_flags = FLAG_NONE;
   for (i = 0; i < mpvparse->ext_count; ++i) {
-    offset = mpvparse->ext_offsets[i];
+    packet.type = GST_MPEG_VIDEO_PACKET_EXTENSION;
+    packet.offset = mpvparse->ext_offsets[i];
+    packet.size = (gint) size - mpvparse->ext_offsets[i];
     mpvparse->config_flags |= FLAG_MPEG2;
-    if (offset < size) {
-      if (gst_mpeg_video_parse_sequence_extension (&mpvparse->sequenceext,
-              map.data, size, offset)) {
-        GST_LOG_OBJECT (mpvparse, "Read Sequence Extension");
-        mpvparse->config_flags |= FLAG_SEQUENCE_EXT;
-      } else
-          if (gst_mpeg_video_parse_sequence_display_extension
-          (&mpvparse->sequencedispext, map.data, size, offset)) {
-        GST_LOG_OBJECT (mpvparse, "Read Sequence Display Extension");
-        mpvparse->config_flags |= FLAG_SEQUENCE_DISPLAY_EXT;
+    if (packet.size > 1) {
+      switch (packet.data[packet.offset] >> 4) {
+        case GST_MPEG_VIDEO_PACKET_EXT_SEQUENCE:
+          if (gst_mpeg_video_packet_parse_sequence_extension (&packet,
+                  &mpvparse->sequenceext)) {
+            GST_LOG_OBJECT (mpvparse, "Read Sequence Extension");
+            mpvparse->config_flags |= FLAG_SEQUENCE_EXT;
+            mpvparse->seqext_updated = TRUE;
+          }
+          break;
+        case GST_MPEG_VIDEO_PACKET_EXT_SEQUENCE_DISPLAY:
+          if (gst_mpeg_video_packet_parse_sequence_display_extension (&packet,
+                  &mpvparse->sequencedispext)) {
+            GST_LOG_OBJECT (mpvparse, "Read Sequence Display Extension");
+            mpvparse->config_flags |= FLAG_SEQUENCE_DISPLAY_EXT;
+            mpvparse->seqdispext_updated = TRUE;
+          }
+          break;
+        case GST_MPEG_VIDEO_PACKET_EXT_QUANT_MATRIX:
+          if (gst_mpeg_video_packet_parse_quant_matrix_extension (&packet,
+                  &mpvparse->quantmatrext)) {
+            GST_LOG_OBJECT (mpvparse, "Read Quantization Matrix Extension");
+            mpvparse->quantmatrext_updated = TRUE;
+          }
+          break;
       }
     }
   }
@@ -318,11 +369,11 @@ gst_mpegv_parse_process_config (GstMpegvParse * mpvparse, GstBuffer * buf,
   /* trigger src caps update */
   mpvparse->update_caps = TRUE;
 
-  gst_buffer_unmap (buf, &map);
-
   return TRUE;
 }
 
+/* FIXME : Move these functions to libgstcodecparser for usage by
+ * more elements/code */
 #ifndef GST_DISABLE_GST_DEBUG
 static const gchar *
 picture_start_code_name (guint8 psc)
@@ -382,29 +433,32 @@ picture_type_name (guint8 pct)
 #endif /* GST_DISABLE_GST_DEBUG */
 
 static void
-parse_picture_extension (GstMpegvParse * mpvparse, GstBuffer * buf, guint off)
+parse_packet_extension (GstMpegvParse * mpvparse, GstMapInfo * info, guint off)
 {
-  GstMpegVideoPictureExt ext;
-  GstMapInfo map;
+  GstMpegVideoPacket packet;
 
-  gst_buffer_map (buf, &map, GST_MAP_READ);
+  packet.data = info->data;
+  packet.type = GST_MPEG_VIDEO_PACKET_EXTENSION;
+  packet.offset = off;
+  packet.size = info->size - off;
 
-  if (gst_mpeg_video_parse_picture_extension (&ext, map.data, map.size, off)) {
+  /* FIXME : WE ARE ASSUMING IT IS A *PICTURE* EXTENSION */
+  if (gst_mpeg_video_packet_parse_picture_extension (&packet,
+          &mpvparse->picext)) {
     mpvparse->frame_repeat_count = 1;
 
-    if (ext.repeat_first_field) {
+    if (mpvparse->picext.repeat_first_field) {
       if (mpvparse->sequenceext.progressive) {
-        if (ext.top_field_first)
+        if (mpvparse->picext.top_field_first)
           mpvparse->frame_repeat_count = 5;
         else
           mpvparse->frame_repeat_count = 3;
-      } else if (ext.progressive_frame) {
+      } else if (mpvparse->picext.progressive_frame) {
         mpvparse->frame_repeat_count = 2;
       }
     }
+    mpvparse->picext_updated = TRUE;
   }
-
-  gst_buffer_unmap (buf, &map);
 }
 
 /* caller guarantees at least start code in @buf at @off */
@@ -412,16 +466,14 @@ parse_picture_extension (GstMpegvParse * mpvparse, GstBuffer * buf, guint off)
  * otherwise returns TRUE if code terminates preceding frame */
 static gboolean
 gst_mpegv_parse_process_sc (GstMpegvParse * mpvparse,
-    GstBuffer * buf, gint off, guint8 code)
+    GstMapInfo * info, gint off, GstMpegVideoPacket * packet)
 {
-  gboolean ret = FALSE, packet = TRUE;
+  gboolean ret = FALSE, checkconfig = TRUE;
 
-  g_return_val_if_fail (buf && gst_buffer_get_size (buf) >= 4, FALSE);
+  GST_LOG_OBJECT (mpvparse, "process startcode %x (%s) offset:%d", packet->type,
+      picture_start_code_name (packet->type), off);
 
-  GST_LOG_OBJECT (mpvparse, "process startcode %x (%s)", code,
-      picture_start_code_name (code));
-
-  switch (code) {
+  switch (packet->type) {
     case GST_MPEG_VIDEO_PACKET_PICTURE:
       GST_LOG_OBJECT (mpvparse, "startcode is PICTURE");
       /* picture is aggregated with preceding sequence/gop, if any.
@@ -449,39 +501,45 @@ gst_mpegv_parse_process_sc (GstMpegvParse * mpvparse,
       break;
     case GST_MPEG_VIDEO_PACKET_EXTENSION:
       GST_LOG_OBJECT (mpvparse, "startcode is VIDEO PACKET EXTENSION");
-      parse_picture_extension (mpvparse, buf, off);
+      parse_packet_extension (mpvparse, info, off);
       if (mpvparse->ext_count < G_N_ELEMENTS (mpvparse->ext_offsets))
         mpvparse->ext_offsets[mpvparse->ext_count++] = off;
-      /* fall-through */
+      checkconfig = FALSE;
+      break;
     default:
-      packet = FALSE;
+      if (GST_MPEG_VIDEO_PACKET_IS_SLICE (packet->type)) {
+        mpvparse->slice_count++;
+        if (mpvparse->slice_offset == 0)
+          mpvparse->slice_offset = off - 4;
+      }
+      checkconfig = FALSE;
       break;
   }
 
   /* set size to avoid processing config again */
-  if (mpvparse->seq_offset >= 0 && off != mpvparse->seq_offset &&
-      !mpvparse->seq_size && packet) {
+  if (checkconfig && mpvparse->seq_offset >= 0 && off != mpvparse->seq_offset &&
+      !mpvparse->seq_size) {
     /* should always be at start */
     g_assert (mpvparse->seq_offset <= 4);
-    gst_mpegv_parse_process_config (mpvparse, buf, off - mpvparse->seq_offset);
+    gst_mpegv_parse_process_config (mpvparse, info, off - mpvparse->seq_offset);
     mpvparse->seq_size = off - mpvparse->seq_offset;
   }
 
   /* extract some picture info if there is any in the frame being terminated */
   if (ret && mpvparse->pic_offset >= 0 && mpvparse->pic_offset < off) {
-    GstMapInfo map;
+    GstMpegVideoPacket header;
 
-    gst_buffer_map (buf, &map, GST_MAP_READ);
-    if (gst_mpeg_video_parse_picture_header (&mpvparse->pichdr,
-            map.data, map.size, mpvparse->pic_offset))
+    header.data = info->data;
+    header.type = GST_MPEG_VIDEO_PACKET_PICTURE;
+    header.offset = mpvparse->pic_offset;
+    header.size = info->size - mpvparse->pic_offset;
+    if (gst_mpeg_video_packet_parse_picture_header (&header, &mpvparse->pichdr))
       GST_LOG_OBJECT (mpvparse, "picture_coding_type %d (%s), ending"
           "frame of size %d", mpvparse->pichdr.pic_type,
           picture_type_name (mpvparse->pichdr.pic_type), off - 4);
     else
       GST_LOG_OBJECT (mpvparse, "Couldn't parse picture at offset %d",
           mpvparse->pic_offset);
-
-    gst_buffer_unmap (buf, &map);
   }
 
   return ret;
@@ -508,6 +566,7 @@ static GstFlowReturn
 gst_mpegv_parse_handle_frame (GstBaseParse * parse,
     GstBaseParseFrame * frame, gint * skipsize)
 {
+  GstFlowReturn flowret = GST_FLOW_OK;
   GstMpegvParse *mpvparse = GST_MPEGVIDEO_PARSE (parse);
   GstBuffer *buf = frame->buffer;
   gboolean ret = FALSE;
@@ -554,7 +613,7 @@ retry:
   /* note: initial start code is assumed at offset 0 by subsequent code */
 
   /* examine start code, see if it looks like an initial start code */
-  if (gst_mpegv_parse_process_sc (mpvparse, buf, 4, packet.type)) {
+  if (gst_mpegv_parse_process_sc (mpvparse, &map, 4, &packet)) {
     /* found sc */
     GST_LOG_OBJECT (mpvparse, "valid start code found");
     mpvparse->last_sc = 0;
@@ -599,7 +658,7 @@ next:
     }
   } else {
     /* decide whether this startcode ends a frame */
-    ret = gst_mpegv_parse_process_sc (mpvparse, buf, off + 4, packet.type);
+    ret = gst_mpegv_parse_process_sc (mpvparse, &map, off + 4, &packet);
   }
 
   if (!ret)
@@ -616,10 +675,16 @@ exit:
     res = gst_mpegv_parse_parse_frame (parse, frame);
     if (res == GST_BASE_PARSE_FLOW_DROPPED)
       frame->flags |= GST_BASE_PARSE_FRAME_FLAG_DROP;
-    return gst_base_parse_finish_frame (parse, frame, off);
+    flowret = gst_base_parse_finish_frame (parse, frame, off);
+    /* Reset local information */
+    mpvparse->seqhdr_updated = FALSE;
+    mpvparse->seqext_updated = FALSE;
+    mpvparse->seqdispext_updated = FALSE;
+    mpvparse->picext_updated = FALSE;
+    mpvparse->quantmatrext_updated = FALSE;
   }
 
-  return GST_FLOW_OK;
+  return flowret;
 }
 
 static void
@@ -654,22 +719,46 @@ gst_mpegv_parse_update_src_caps (GstMpegvParse * mpvparse)
       "parsed", G_TYPE_BOOLEAN, TRUE, NULL);
 
   if (mpvparse->sequencehdr.width > 0 && mpvparse->sequencehdr.height > 0) {
-    gst_caps_set_simple (caps, "width", G_TYPE_INT, mpvparse->sequencehdr.width,
-        "height", G_TYPE_INT, mpvparse->sequencehdr.height, NULL);
+    GstMpegVideoSequenceDisplayExt *seqdispext;
+    gint width, height;
+
+    width = mpvparse->sequencehdr.width;
+    height = mpvparse->sequencehdr.height;
+
+    if (mpvparse->config_flags & FLAG_SEQUENCE_DISPLAY_EXT) {
+      seqdispext = &mpvparse->sequencedispext;
+
+      if (seqdispext->display_horizontal_size <= width
+          && seqdispext->display_vertical_size <= height) {
+        width = seqdispext->display_horizontal_size;
+        height = seqdispext->display_vertical_size;
+        GST_INFO_OBJECT (mpvparse,
+            "stream has display extension: display_width=%d display_height=%d",
+            width, height);
+      }
+    }
+    gst_caps_set_simple (caps, "width", G_TYPE_INT, width,
+        "height", G_TYPE_INT, height, NULL);
   }
 
   /* perhaps we have a framerate */
-  if (mpvparse->fps_num > 0 && mpvparse->fps_den > 0 &&
-      (!s || !gst_structure_has_field (s, "framerate"))) {
+  {
     gint fps_num = mpvparse->fps_num;
     gint fps_den = mpvparse->fps_den;
-    GstClockTime latency = gst_util_uint64_scale (GST_SECOND, fps_den, fps_num);
+    GstClockTime latency;
 
-    gst_caps_set_simple (caps, "framerate",
-        GST_TYPE_FRACTION, fps_num, fps_den, NULL);
-    gst_base_parse_set_frame_rate (GST_BASE_PARSE (mpvparse),
-        fps_num, fps_den, 0, 0);
-    gst_base_parse_set_latency (GST_BASE_PARSE (mpvparse), latency, latency);
+    /* upstream overrides */
+    if (s && gst_structure_has_field (s, "framerate"))
+      gst_structure_get_fraction (s, "framerate", &fps_num, &fps_den);
+
+    if (fps_den > 0 && fps_num > 0) {
+      gst_caps_set_simple (caps, "framerate",
+          GST_TYPE_FRACTION, fps_num, fps_den, NULL);
+      gst_base_parse_set_frame_rate (GST_BASE_PARSE (mpvparse),
+          fps_num, fps_den, 0, 0);
+      latency = gst_util_uint64_scale (GST_SECOND, fps_den, fps_num);
+      gst_base_parse_set_latency (GST_BASE_PARSE (mpvparse), latency, latency);
+    }
   }
 
   /* or pixel-aspect-ratio */
@@ -775,6 +864,10 @@ gst_mpegv_parse_parse_frame (GstBaseParse * parse, GstBaseParseFrame * frame)
     GST_BUFFER_DURATION (buffer) = 0;
   }
 
+  if (mpvparse->pic_offset > 4) {
+    gst_base_parse_set_ts_at_offset (parse, mpvparse->pic_offset - 4);
+  }
+
   if (mpvparse->frame_repeat_count
       && GST_CLOCK_TIME_IS_VALID (GST_BUFFER_DURATION (buffer))) {
     GST_BUFFER_DURATION (buffer) =
@@ -795,6 +888,13 @@ gst_mpegv_parse_pre_push_frame (GstBaseParse * parse, GstBaseParseFrame * frame)
 {
   GstMpegvParse *mpvparse = GST_MPEGVIDEO_PARSE (parse);
   GstTagList *taglist;
+  GstMpegVideoMeta *meta;
+  GstMpegVideoSequenceHdr *seq_hdr = NULL;
+  GstMpegVideoSequenceExt *seq_ext = NULL;
+  GstMpegVideoSequenceDisplayExt *disp_ext = NULL;
+  GstMpegVideoPictureHdr *pic_hdr = NULL;
+  GstMpegVideoPictureExt *pic_ext = NULL;
+  GstMpegVideoQuantMatrixExt *quant_ext = NULL;
 
   /* tag sending done late enough in hook to ensure pending events
    * have already been sent */
@@ -818,6 +918,37 @@ gst_mpegv_parse_pre_push_frame (GstBaseParse * parse, GstBaseParseFrame * frame)
   /* usual clipping applies */
   frame->flags |= GST_BASE_PARSE_FRAME_FLAG_CLIP;
 
+  if (mpvparse->send_mpeg_meta) {
+    GstBuffer *buf;
+
+    if (mpvparse->seqhdr_updated)
+      seq_hdr = &mpvparse->sequencehdr;
+    if (mpvparse->seqext_updated)
+      seq_ext = &mpvparse->sequenceext;
+    if (mpvparse->seqdispext_updated)
+      disp_ext = &mpvparse->sequencedispext;
+    if (mpvparse->picext_updated)
+      pic_ext = &mpvparse->picext;
+    if (mpvparse->quantmatrext_updated)
+      quant_ext = &mpvparse->quantmatrext;
+    pic_hdr = &mpvparse->pichdr;
+
+    GST_DEBUG_OBJECT (mpvparse,
+        "Adding GstMpegVideoMeta (slice_count:%d, slice_offset:%d)",
+        mpvparse->slice_count, mpvparse->slice_offset);
+
+    if (frame->out_buffer) {
+      buf = frame->out_buffer = gst_buffer_make_writable (frame->out_buffer);
+    } else {
+      buf = frame->buffer = gst_buffer_make_writable (frame->buffer);
+    }
+
+    meta =
+        gst_buffer_add_mpeg_video_meta (buf, seq_hdr, seq_ext, disp_ext,
+        pic_hdr, pic_ext, quant_ext);
+    meta->num_slices = mpvparse->slice_count;
+    meta->slice_offset = mpvparse->slice_offset;
+  }
   return GST_FLOW_OK;
 }
 
@@ -835,10 +966,14 @@ gst_mpegv_parse_set_caps (GstBaseParse * parse, GstCaps * caps)
 
   if ((value = gst_structure_get_value (s, "codec_data")) != NULL
       && (buf = gst_value_get_buffer (value))) {
+    GstMapInfo map;
+    gst_buffer_map (buf, &map, GST_MAP_READ);
     /* best possible parse attempt,
      * src caps are based on sink caps so it will end up in there
      * whether sucessful or not */
-    gst_mpegv_parse_process_config (mpvparse, buf, gst_buffer_get_size (buf));
+    mpvparse->seq_offset = 4;
+    gst_mpegv_parse_process_config (mpvparse, &map, gst_buffer_get_size (buf));
+    gst_buffer_unmap (buf, &map);
     gst_mpegv_parse_reset_frame (mpvparse);
   }
 
